import Tkinter as tk
import pydicom as dicom
import processDicom as pydicom_Tkinter
import Tkconstants, tkFileDialog
import tkMessageBox
import tkSimpleDialog
import glob
import os
import csv
from os.path import dirname, join
import matplotlib
matplotlib.use('TkAgg')
import matplotlib.pyplot as plt
import matplotlib.image as mpimg
from matplotlib.backends.backend_tkagg import (
    FigureCanvasTkAgg, NavigationToolbar2Tk)
from matplotlib.figure import Figure
from matplotlib.patches import Polygon
import numpy as np
from sys import getsizeof

display = False


class AnnotationTools(tk.Frame):
    def __init__(self, parent, figureNo, row, col):
        tk.Frame.__init__(self, parent)
        global app

        self.annotationNumber = figureNo
        self.annotated = False
        self.messageShown = False
        self.annotationFrame = tk.Frame(parent)
        self.f = app.getFigure(figureNo).getFigure()
        self.a = app.getFigure(figureNo).getAxes()
        self.allow = None
        self.annotationCoords = []
        self.polygonPlotArray = []



        self.annotationFrame.grid(row=row, column=col, sticky="nsew")

        self.secondRow = tk.Frame(parent)
        self.secondRow.grid(row=(row + 1), column=col, sticky="nsew")

        self.annotate = tk.Button(self.annotationFrame, text="Annotation Mode", state="normal", width="15", command=lambda:self.selectAnnotation())
        self.annotate.pack(side="left", padx=(2,2))

        self.saveAnnotation = tk.Button(self.annotationFrame, text="Save Annotation", state="disabled", width="15", command=self.annotationToFile)
        self.saveAnnotation.pack(side="left", padx=(2,2))

        self.drawPolygon = tk.Button(self.annotationFrame, text="Draw Polygon", state="disabled", width="15", command=self.drawPolygonPatch)
        self.drawPolygon.pack(side="left", padx=(2,2))

        self.clearPlots = tk.Button(self.secondRow, text="Clear Plots", state="disabled", width="15", command=lambda:self.removePlots(False))
        self.clearPlots.pack(side="left", padx=(2,2))

        self.deleteAnnotations = tk.Button(self.secondRow, text="Delete Annotations", state="disabled", width="15", command=lambda:self.removePlots(True))
        self.deleteAnnotations.pack(side="left", padx=(2,2))

        self.cancel = tk.Button(self.secondRow, text="Cancel", state="disabled", width="15", command=self.cancelAnnotation)
        self.cancel.pack(side="left", padx=(2,2))

        self.changeDirectory = tk.Button(self.secondRow, text="Change Directory", state="normal", width="15", command=lambda:self.newFileDirectory(parent))
        self.changeDirectory.pack(side="right", padx=(2,2))
        #
        # self.syncPoints = tk.Button(self.secondRow, text="Sync Points", state="normal", width="15", command=self.printDicom)
        # self.syncPoints.pack(side="right", padx=(2,2))
        #

    def selectAnnotation(self):
        global app
        self.annotated = True
        self.annotationCoords = []
        self.polygonPlotArray = []

        if self.annotated:
            if self.messageShown == False:
                tkMessageBox.showinfo("Annotation",
                "Please select the coordinates you wish to draw around then use 'save annotation' to save your points. If you wish to draw around the area, use 'draw polygon' when you have fnished plotting the points, then save the annotation")

            self.allow = self.f.canvas.mpl_connect('button_press_event', self.coordinatesClick)
            self.messageShown = True
            self.annotate.config(state='disabled')
            app.getNavigation().changeButtonState('disabled')


            self.cancel.config(state='normal')

    def coordinatesClick(self, event):
        coords = (str(event.xdata) + "," + str(event.ydata))
        self.a.scatter(event.xdata, event.ydata, c='y', s=10, zorder=2)
        self.a.figure.canvas.draw_idle()
        self.annotationCoords.append(coords)
        self.setPolygonPlot(event.xdata, event.ydata)
        self.drawPolygon.config(state='normal')
        self.saveAnnotation.config(state='normal')
        self.clearPlots.config(state='normal')


    def annotationToFile(self):
        global app
        patientID = app.getCurrentDicom(self.annotationNumber).getPatientID()
        scanType = app.getCurrentDicom(self.annotationNumber).getScanType()
        slice = app.getCurrentDicom(self.annotationNumber).getSliceNo()

        app.getCurrentDicom(self.annotationNumber).addPoints(self.annotationCoords)
        app.getCurrentDicom(self.annotationNumber).addPolygon(self.getPolygonPlot())
        annotationInfo = str(patientID + '-' + scanType + '-')
        filename = os.path.join(app.getDirectory(self.annotationNumber), annotationInfo + "annotation.csv")
        if self.annotated == True:
            self.f.canvas.mpl_disconnect(self.allow)
            answer = tkSimpleDialog.askstring('PI-RADS Score', 'Level of suspision on PI-RADS scale')
            csvAnnotator = CsvOperator(filename)
            csvAnnotator.writePointsToCSV(self.annotationCoords, patientID, scanType, slice, answer, self)
        del self.annotationCoords [:]
        self.annotated = False
        self.saveAnnotation.config(state='disabled')
        self.drawPolygon.config(state='disabled')
        self.clearPolyPlot()
        self.annotate.config(state='normal')
        app.getNavigation().changeButtonState('normal')
        self.cancel.config(state='disabled')

    def cancelAnnotation(self):
        global app
        self.f.canvas.mpl_disconnect(self.allow)
        self.saveAnnotation.config(state='disabled')
        self.drawPolygon.config(state='disabled')
        self.annotate.config(state='normal')
        app.getNavigation().changeButtonState('normal')
        self.cancel.config(state='disabled')
        self.removePlots(False)
        plt.cla()
        app.getFigure(self.annotationNumber).changeDicom(app.getCurrentDicom(self.annotationNumber), True)

    def removePlots(self, complete):
        global app
        self.annotationCoords = []
        self.polygonPlotArray = []

        self.a.clear()
        self.a.figure.canvas.draw_idle()
        if complete:
            app.getCurrentDicom(self.annotationNumber).deletePlots('all')
        app.getFigure(self.annotationNumber).changeDicom(app.getCurrentDicom(self.annotationNumber), False)


    def drawPolygonPatch(self):
        global app
        polygonShape = Polygon(self.getPolygonPlot(), closed=True, figure=self.f, zorder=3, fill=False, edgecolor="yellow")
        app.getFigure(self.annotationNumber).addPolygon(polygonShape)

    def enableDelete(self):
        self.deleteAnnotations.config(state='normal')

    def setPolygonPlot(self,x,y):
        self.polygonPlotArray.append([x,y])

    def getPolygonPlot(self):
        return self.polygonPlotArray

    def clearPolyPlot(self):
        self.polygonPlotArray = []

    def printDicom(self):
        global app
        print app.getCurrentDicom(1).printSelf()

    def newFileDirectory(self, parent):
        global app
        newBrowse = FolderBrowser(parent)
        if newBrowse.browseFolder():
            app.changeArray(self.annotationNumber, newBrowse.getDicomArray(), newBrowse.getDirectory())


class Navigation(tk.Frame):
    def __init__(self, parent, figure, number, row, col, app):
        tk.Frame.__init__(self, parent)

        self.bFrame = tk.Frame(parent)
        #The frame within which the naviagtion lies
        self.bFrame.grid(row=row, column=col, sticky="nsew")

        #for use within the classes methods, allows us to update the figure depending on interactions with the navigation
        self.parentFigure = figure

        self.previousButton = tk.Button(self.bFrame, width="10", text="Previous", command=lambda:self.prevDicom(figure, app))
        self.previousButton.pack(side="left", padx=(0,10), pady=(0,5))

        self.nextButton = tk.Button(self.bFrame, width="10", text="Next", command=lambda:self.nextDicom(figure, app))
        self.nextButton.pack(side="left",  pady=(0,5))

        #functionality that will only be usable within the first screen.
        if number == 1 and app.getScreenStatus():
            self.dualScreen = tk.Button(self.bFrame, width="10", text="Dual Screen", command=lambda:self.setUpDual(app))
            self.dualScreen.pack(side="right",  padx=(5,5), pady=(0,5))
            self.sync = tk.Button(self.bFrame, width="10", text="Sync", state="disabled", command=lambda:self.syncScans(app))
            self.sync.pack(side="right",  padx=(5,5), pady=(0,5))

        elif number == 1 and app.getScreenStatus() == False:
            self.sync = tk.Button(self.bFrame, width="10", text="Sync", state="normal", command=lambda:self.syncScans(app))
            self.sync.pack(side="right",  padx=(5,5), pady=(0,5))



        self.navNumber = number


    def nextDicom(self, figure, app):
        #As each dicom object in the array is aware of what position it is in the stack,
        #we can use this to navigate back by calling the getNext() method.
        #special for -1, which means we are position 0 and trying to navigate to
        # the last position in the stack
        dicom = app.getCurrentDicom(self.navNumber)
        next = dicom.getNext()
        array = app.getDicomArray(self.navNumber)
        if next < len(array):
            # Change the currently displayed dicom
            app.setCurrentDicom(array[next], self.navNumber)
            # Clear all annotations on the figure
            figure.changeClear()
            # Change the figure to display the next slice in the stack
            figure.changeDicom(array[next], True)

        else:
            # for cases where we are on the last slice in the stack
            app.setCurrentDicom(array[0], self.navNumber)
            figure.changeClear()
            figure.changeDicom(array[0], True)


    def prevDicom(self, figure, app):
        #As each dicom object in the array is aware of what position it is in the stack,
        #we can use this to navigate back by calling the getPrevious() method.
        #special for -1, which means we are position 0 and trying to navigate to
        # the last position in the stack
        dicom = app.getCurrentDicom(self.navNumber)
        previous = dicom.getPrevious()
        array = app.getDicomArray(self.navNumber)

        if previous > -1:
            app.setCurrentDicom(array[previous], self.navNumber)
            figure.changeClear()
            figure.changeDicom(array[previous], True)


        else:
            app.setCurrentDicom(array[len(array) - 1], self.navNumber)
            figure.changeClear()
            figure.changeDicom(array[len(array) - 1], True)


    def changeButtonState(self, type):
            self.previousButton.config(state=type)
            self.nextButton.config(state=type)

    def setUpDual(self, app):
        app.setupDual()

    def showSyncMode(self, app):
        self.forgetSync()
        self.syncMode = tk.Button(self.bFrame, text="Synchronise Scans", state="normal", command=lambda:self.showSyncedSlice(app))
        self.syncMode.pack(side="right",  padx=(5,5), pady=(0,5))

    def showSyncedSlice(self, app):
        syncedChange = app.getCurrentDicom(1).getSynchronised()
        if syncedChange != None:
            dicomFile = app.getSpecificDicom(2, syncedChange)
            app.getFigure(2).changeDicom(dicomFile, True)
            app.setCurrentDicom(dicomFile, 2)
        else:
            slice = app.getCurrentDicom(1).getSliceNo()
            tkMessageBox.showinfo("Sync Error",
            "Slice " + str(slice) + " has no comparable slices in the secondary stack")

    def syncScans(self, app):

        # here I get the two stacks
        stackA = app.getDicomArray(1)
        stackB = app.getDicomArray(2)

        if stackA[0].getFrameOfReference() == stackB[0].getFrameOfReference():
            # get the first and second slices in the stacks as reference
            aSliceZero = stackA[0].getImagePosition()
            aSliceOne = stackA[1].getImagePosition()

            bSliceZero = stackB[0].getImagePosition()
            bSliceOne = stackB[1].getImagePosition()

            aDirectionRef = np.subtract(aSliceOne,aSliceZero)
            bDirectionRef = np.subtract(bSliceOne,bSliceZero)



            #  some flags set up to check whether the stack goes in the same direction
            #  we assume its true, any deviation and we set to false
            aStackDirection = True
            bStackDirection = True
            aligned = False


            for k in range(0, len(stackA)):

                if k != len(stackA) - 1:
                    # on each loop get the image position of slice k & k +1 in the stack
                    referenceA = stackA[k].getImagePosition()
                    referenceB = stackA[k+1].getImagePosition()

                    # calculate the direction between the two slices
                    directionA = np.subtract(referenceB,referenceA)
                    # then calculate the dot product between the reference direction
                    # (slices 0,1) and the direction calculated above
                    dot = np.dot(directionA, aDirectionRef)
                    # get the norm of the reference direction and the one above
                    same = np.linalg.norm(directionA) * np.linalg.norm(aDirectionRef)


                    # check if these are the same, get the absolute value as some will go into
                    # negative (slight differences between the numbers could drop into exteremly small negative decimals)
                    product = same - dot
                    value = abs(product)
                    # if the value is within a 0.1 range, could be increased, then fine
                    # otherwise, the stack direction is compromised
                    if value < 0.1:
                        continue
                    else:
                        aStackDirection = False

            for k in range(0, len(stackB)):
                if k != len(stackB) - 1:
                    referenceA = stackB[k].getImagePosition()
                    referenceB = stackB[k+1].getImagePosition()

                    directionB = np.subtract(referenceB,referenceA)
                    dot = np.dot(directionB, bDirectionRef)
                    same = np.linalg.norm(directionB) * np.linalg.norm(bDirectionRef)

                    product = same - dot
                    value = abs(product)

                    if value < 0.1:
                        continue
                    else:
                        bStackDirection = False

            if aStackDirection & bStackDirection:
                # here we check for allignment between the two stacks.
                alignedDot = np.dot(aDirectionRef, bDirectionRef)
                alignedDist = np.linalg.norm(aDirectionRef) * np.linalg.norm(bDirectionRef)

                product = alignedDist - alignedDot
                value = abs(product)
                if value < 0.1:
                    aligned = True

            if aligned:

                #the first point in the A stack, what we will be using to project points along the line
                pointZero = np.add(stackA[0].getSliceLocation(), (0 * aDirectionRef))
                synced = False

                #loop over the A stack, get each position, calculate its point along the line
                for i in range(0, len(stackA)):
                    if i > 0 and (i != len(stackA) - 1):
                        plusOne = np.subtract(stackA[i+1].getSliceLocation(),stackA[i].getSliceLocation())
                        minusOne = np.subtract(stackA[i-1].getSliceLocation(),stackA[i].getSliceLocation())
                    #here we calculate A[i] position on the line
                    pointA = np.add(stackA[0].getSliceLocation(), (i * aDirectionRef))
                    bestFit = None
                    dist = None
                    for k in range(0, len(stackB)):
                        # loop through the B stack, calculate the point on the B line below
                        pointB = np.add(stackB[0].getSliceLocation(), (k * bDirectionRef))
                        #the first half of the algorithm, this corresponts to A[0] + dot(B[k]-A[0],d) * d
                        # where pointZero is A[0], then we get the dot product of (pointB - A[0]) and d, the direction from slice 0 -> 1
                        #and divide that by the norm of d
                        bProject = np.dot(np.subtract(pointB,pointZero),aDirectionRef) / np.linalg.norm(aDirectionRef)

                        #here we have the second half of the algrithm  d / norm(d)^2
                        normD = aDirectionRef/(np.linalg.norm(aDirectionRef))

                        #multiply the two together
                        mult = bProject*normD

                        #add to point A[0] (the first point on the line)
                        Bpk = np.add(pointZero, mult)

                        #get the norm of pointA minus the projected point to get its position on the line
                        normPoint = abs(np.linalg.norm(np.subtract(pointA, Bpk)))
                        compare = abs(np.linalg.norm(aDirectionRef)/2)


                        if normPoint < compare:
                            synced = True
                            if bestFit == None:
                                bestFit = k
                                dist = normPoint
                            elif dist > normPoint:
                                bestFit = k
                                dist = normPoint
                            break
                        elif i > 0 and i < len(stackA) - 1:
                            synced = True
                            secondCompare = max(np.linalg.norm(plusOne), np.linalg.norm(minusOne))/2
                            if normPoint < secondCompare:
                                if bestFit == None:
                                    bestFit = k
                                    dist = normPoint
                                elif dist > normPoint:
                                    bestFit = k
                                    dist = normPoint
                    if bestFit != None:
                        stackA[i].setSynchronised(bestFit)
                        stackB[bestFit].setSynchronised(i)

                if synced:
                    tkMessageBox.showinfo("Synchronise Completion",
                    "The stacks have been sychronised")

                    self.showSyncMode(app)
                else:
                    tkMessageBox.showinfo("Synchronise Error",
                    "The stacks could not be synchronised")

            else:
                    tkMessageBox.showinfo("Synchronise Error",
                    "The stacks are not alligned")
        else:
                tkMessageBox.showinfo("Synchronise Error",
                "The two stacks are not comparable, check the PatientID in each stack")




    def forgetDual(self):
        self.dualScreen.pack_forget()
        self.sync.config(state="normal")

    def forgetSync(self):
        self.sync.pack_forget()



class CsvOperator():
    #a class for all cvs operations we need to do
    def __init__(self, csvfile):

        self.csvFile = csvfile
        self.annotationWriter = None

    def writePointsToCSV(self, pointArray, patientID, scanType, slice, answer, parent):
        with open(self.csvFile, 'a') as csvfile:
            self.annotationWriter = csv.writer(csvfile, delimiter=",")
            self.annotationWriter.writerow([patientID] + [scanType] + [slice] + [answer] + ["points"] + [pointArray])
        parent.enableDelete()



    def getPointsFromCoords(self, pointArray):
        for coords in pointArray:
            string = "["+ coords +"]"
            with open(self.csvFile, 'a') as csvfile:
                self.annotationWriter.writerow(string)


class DicomObject():
    def __init__(self,dicomFile, position):
        self.filename = dicomFile
        self.ds = dicom.dcmread(self.filename)
        self.pixel_array = self.ds.pixel_array
        self.patientID = self.ds.PatientID
        self.scanType = self.ds.SeriesDescription
        self.slice = self.ds.InstanceNumber
        self.synchronised = None
        self.stackPosition = position
        self.polygons = []
        self.points = []

    def getPatientID(self):
        return self.patientID

    def getScanType(self):
        return self.scanType

    def printSelf(self):
        # print self.filename
        # print self.scanType
        # print self.ds.PixelSpacing
        # print self.ds.ImageOrientationPatient
        # print self.ds.ImagePositionPatient
        # print self.ds.SliceThickness
        # print self.ds.SliceLocation
        # print self.ds.FrameOfReferenceUID
        # print self.ds.dir('matrix')
        xCoord = None
        yCoord = None
        zCoord = None
        # corresponds to Xxyz (row x direction cosine of image orientation)
        orientA = self.ds.ImageOrientationPatient[:3]

        # corresponds to Yxyz (row y direction cosine of image orientation)
        orientB = self.ds.ImageOrientationPatient[-3:]

        #randomly picked coordinates in the 2d image space
        i = 103.2229437229437
        j = 104.6082251082251
        z = 0

        #as we need X,Y,Z coordinates
        for x in range(0,3):
            #these correspond to the columns in row x
            colOne = orientA[x] * self.ds.PixelSpacing[0] * i
            colTwo = orientB[x] * self.ds.PixelSpacing[1] * j
            colThree = 0 * self.ds.ImagePositionPatient[x]


            if x == 0:
                xCoord = colOne + colTwo + colThree
            elif x == 1:
                yCoord = colOne + colTwo + colThree
            elif x == 2:
                zCoord = colOne + colTwo + colThree

        #the X,Y,Z coordinates for point(i,j)
        patientCoords = [xCoord, yCoord, zCoord]

        print patientCoords

    def getSliceNo(self):
        return self.slice

    def getDicomObj(self):
        return self.ds

    def getFilename(self):
        return self.filename

    def getFrameOfReference(self):
        return self.ds.FrameOfReferenceUID

    def getStackPosition(self):
        return self.stackPosition

    def getPixelArray(self):
        return self.pixel_array

    def getNext(self):
        return self.stackPosition + 1

    def getImagePosition(self):
        return self.ds.ImagePositionPatient

    def getSliceLocation(self):
        return self.ds.SliceLocation

    def setSynchronised(self, sliceNumber):
        self.synchronised = sliceNumber

    def getSynchronised(self):
        return self.synchronised

    def getPixelSpacing(self):
        return self.ds.PixelSpacing

    def getPrevious(self):
        return self.stackPosition - 1

    def addPolygon(self, newPolygonArray):
        self.polygons.append(newPolygonArray)

    def getPolygons(self):
        return self.polygons

    def addPoints(self, newPointsArray):
        new = list(newPointsArray)
        if len(new) > 0:
            self.points.append(new)

    def getPoints(self):
        return self.points

    def addCircle(self, newCircleArray):
        self.circles.append(newCircleArray)


    def drawAnnotations(self, axes, figure):
        if len(self.points) > 0:
            self.drawPoints(axes)
        if len(self.polygons) > 0:
            self.drawPolygon(axes, figure)

    def drawPoints(self, axes):
        for coords in self.points:
            for points in coords:
                point = points.split(',')
                x = float(point[0])
                y = float(point[1])
                axes.scatter(x, y, c='y', s=10, zorder=2)
                axes.figure.canvas.draw_idle()

    def drawPolygon(self, axes, figure):
        for coords in self.polygons:
            polygonShape = Polygon(coords, closed=True, figure=figure, zorder=3, fill=False, edgecolor="yellow")
            axes.add_patch(polygonShape)
            axes.figure.canvas.draw_idle()


    def deletePlots(self, type):
        if type == 'points':
            self.points = []
        elif type == 'polygons':
            self.polygons = []
        elif type == 'all':
            self.points = []
            self.polygons = []





class FolderBrowser(tk.Frame):

    def __init__(self, parent):


        self.dicomArray = []
        self.directory = None

    def browseFolder(self):
        folder = tkFileDialog.askdirectory(initialdir = "/")
        self.directory = folder
        count = 0
        for filename in glob.glob(os.path.join(folder, '*.dcm')):
            self.dicomArray.append(DicomObject(filename, count))
            count = count + 1
        if(len(self.dicomArray) > 0):
            return True
        else:
            return False

    def getDicomArray(self):
        return self.dicomArray

    def getDirectory(self):
        return self.directory


class FigureView(tk.Frame):

    def __init__(self, parent, axes, initialDicom, number, toolRow, toolCol, figRow, figCol, labelRow, labelCol):
        global app
        tk.Frame.__init__(self, parent)
        self.labelFrame = tk.Frame(parent)
        self.labelFrame.grid(row=labelRow, column=labelCol)

        self.toolbarFrane = tk.Frame(parent)
        self.toolbarFrane.grid(row=toolRow, column=toolCol)


        self.idLabel = tk.Label(master=self.labelFrame, text="patientID: " + initialDicom.getPatientID())
        self.idLabel.pack(side="left", padx=(5,5))

        self.sequenceType = tk.Label(master=self.labelFrame, text="Scan Type: " + initialDicom.getScanType())
        self.sequenceType.pack(side="left", padx=(5,5))

        self.sliceNo = tk.Label(master=self.labelFrame, text="Slice Number: " + str(initialDicom.getSliceNo()) + " of " + str(len(app.getDicomArray(number))))
        self.sliceNo.pack(side="left", padx=(5,5))


        self.f = plt.figure()
        self.a = self.f.add_subplot(1,1,1)
        self.imageShow = self.a.imshow(initialDicom.getPixelArray(), cmap=plt.cm.bone, zorder=1)

        self.canvas = FigureCanvasTkAgg(self.f, master=parent)
        self.canvas.get_tk_widget().grid(row=figRow, column=figCol)
        self.canvas._tkcanvas.grid(row=figRow, column=figCol)

        self.toolbar = NavigationToolbar2Tk(self.canvas, self.toolbarFrane)
        self.toolbar.pack()


        self.figureNumber = number

        self.currentAnnotations = []



    def getFigure(self):
        return self.f

    def getAxes(self):
        return self.a

    def changeClear(self):
        self.a.clear()
        self.a.figure.canvas.draw_idle()


    def getCanvas(self):
        return self.canvas

    def changeDicom(self, dicomFile, redraw):
            self.imageShow = self.a.imshow(dicomFile.getPixelArray(), cmap=plt.cm.bone, zorder=1)
            self.a.figure.canvas.draw_idle()
            self.changeAll(dicomFile)
            if redraw:
                dicomFile.drawAnnotations(self.a, self.f)

    def changeAll(self, dicomFile):
        self.idLabel['text'] = "patientID: " + dicomFile.getPatientID()
        self.sequenceType['text'] = "Scan Type:  " + dicomFile.getScanType()
        self.sliceNo['text'] = "Slice Number: " + str(dicomFile.getSliceNo()) + " of " + str(len(app.getDicomArray(self.figureNumber)))

    def addPolygon(self, polygon):
        self.a.add_patch(polygon)
        self.a.figure.canvas.draw_idle()





class Main(tk.Tk):

    def __init__(self):
        # __init__ describes the initialisation of a python class - tk.Tk.__init__ describes the
        # initialisation of the tkinter module
        tk.Tk.__init__(self)
        self.title('Dicom Stack Annotator') #name of the program
        self.container = tk.Frame(self) #setup the container from which the rest of the app is displayed

        self.container.pack(side="top", fill="both", expand=True) # pack the container on screen

        #configure options for the geometry of the application
        self.container.grid_rowconfigure(0, weight=1)
        self.container.grid_columnconfigure(0, weight=1)

        #instanciate the FolderBrowser class, which is the start point of the application
        self.browse = FolderBrowser(self.container)
        #the first thing the user see's, calls the setup method which creates the main body of the application
        self.browse_button = tk.Button(self.container, width="15", text="Open Directory",
                                    command=self.setup)
        self.browse_button.pack()

        #variables to be used by the Main class and other classes in the application.
        self.dicomArray = None
        self.dicomArrayDual = None
        self.currentDicom = None
        self.currentDicomDual = None
        self.navigation = None
        self.navigationTwo = None
        self.annotationTools = None
        self.annotationToolsTwo = None
        self.figureOne = None
        self.figureTwo = None
        self.dualBrowse = None
        self.aStackFileDirectory = None
        self.bStackFileDirectory = None
        self.synced = False
        self.syncMode = False
        self.singleScreen = True




    def setup(self):
        if(self.browse.browseFolder()):
            self.browse_button.pack_forget()
            self.state('zoomed')
            self.dicomArray = self.browse.getDicomArray()
            self.aStackFileDirectory = self.browse.getDirectory()
            self.setCurrentDicom(self.dicomArray[0], 1)
            self.figureOne = FigureView(self.container,1,self.getCurrentDicom(1),1,1,0,2,0,0,0)
            self.navigation = Navigation(self.container, self.figureOne, 1, 3, 0, self)
            self.annotationTools = AnnotationTools(self.container, 1, 4, 0)
        else:
            tkMessageBox.showinfo("Setup Error",
            "There are no DICOM files in this directory, try again")


    def setCurrentDicom(self, dicomObject, number):
        if number == 1:
            self.currentDicom = dicomObject
        if number == 2:
            self.currentDicomDual = dicomObject


    def setDualDicom(self, dicomObject):
        self.currentDicomDual = dicomObject

    def getCurrentDicom(self, number):
        if number == 1:
            return self.currentDicom
        elif number == 2:
            return self.currentDicomDual

    def getDicomArray(self, number):
        if number == 1:
            return self.dicomArray
        elif number == 2:
            return self.dicomArrayDual

    def getFigure(self, number):
        if number == 1:
            return self.figureOne
        if number == 2:
            return self.figureTwo

    def getDirectory(self, number):
        if number == 1:
            return self.aStackFileDirectory
        if number == 2:
            return self.bStackFileDirectory

    def getSpecificDicom(self, number, dicom):
        if number == 1:
            return self.dicomArray[dicom]
        if number == 2:
            return self.dicomArrayDual[dicom]

    def changeArray(self, number, array, directory):
        if number == 1:
            self.dicomArray = array
            self.setCurrentDicom(self.dicomArray[0], number)
            self.aStackFileDirectory = directory
            self.figureOne = FigureView(self.container,1,self.getCurrentDicom(1),1,1,0,2,0,0,0)
            self.navigation = Navigation(self.container, self.figureOne, 1, 3, 0, self)
            self.annotationTools = AnnotationTools(self.container, 1, 4, 0)
        if number == 2:
            self.dicomArrayDual = array
            self.setCurrentDicom(self.dicomArrayDual[0], number)
            self.bStackFileDirectory = directory
            self.figureTwo = FigureView(self.container,2,self.getCurrentDicom(2),2,1,1,2,1,0,1)
            self.navigationTwo = Navigation(self.container, self.figureTwo, 2, 3, 1, self)
            self.annotationToolsTwo = AnnotationTools(self.container, 2, 4, 1)



    def getNavigation(self):
        return self.navigation

    def setupDual(self):
        self.dualBrowse = FolderBrowser(self.container)
        if(self.dualBrowse.browseFolder()):
            self.singleScreen = False
            self.dicomArrayDual = self.dualBrowse.getDicomArray()
            self.bStackFileDirectory = self.dualBrowse.getDirectory()
            self.setCurrentDicom(self.dicomArrayDual[0], 2)
            self.navigation.forgetDual()
            self.figureTwo = FigureView(self.container,2,self.getCurrentDicom(2),2,1,1,2,1,0,1)
            self.navigationTwo = Navigation(self.container, self.figureTwo, 2, 3, 1, self)
            self.annotationToolsTwo = AnnotationTools(self.container, 2, 4, 1)
        else:
            tkMessageBox.showinfo("Setup Error",
            "There are no DICOM files in this directory, try again")

    def setSynced(self, sync):
        self.synced = sync

    def getSynced(self, sync):
        return self.synced

    def setSyncMode(self, mode):
        self.syncMode = mode

    def getSyncMode(self, mode):
        return self.syncMode

    def getScreenStatus(self):
        return self.singleScreen


app = Main()
app.mainloop()
